import 'dart:convert';
import 'dart:io';
import 'package:pioneer/pioneer.dart';
import 'package:pioneer/src/json_request.dart';

class Application {
  List<Middleware> _middlewares;
  List<Route> _routes;

  Application({
    List<Middleware> middlewares = const [],
    List<Route> routes = const [],
    Map<String, List<dynamic>> converters = const {}
  }) {
    this._middlewares = []..addAll(middlewares);
    this._routes = []..addAll(routes);
  }

  void run() async {
    HttpServer server = await HttpServer.bind(InternetAddress.anyIPv4, 8080);
    await for (var httpRequest in server) {
      try {
        var data = jsonDecode(
          await utf8.decoder.bind(httpRequest).join()
        );
        JsonRequest jsonRequest = new JsonRequest(httpRequest, data);
        JsonResponse response = this.handleRequest(jsonRequest);
        
        if (response != null) {
          response.write(httpRequest.response);
        } else {
          (new JsonResponse(statusCode: HttpStatus.notFound)).write(
            httpRequest.response
          );
        }
      } catch (exception) {
        (new JsonResponse(statusCode: HttpStatus.internalServerError)).write(
          httpRequest.response
        );
      }
    }
              
    /*
    print(request.protocolVersion);
    print(request.headers['connection']);
    print(request.requestedUri);
    print(request.uri);
    */
  }

  JsonResponse handleRequest(JsonRequest request) {
    for (var route in this._routes) {
      Map<Symbol, dynamic> parameters = route.matchAndExtractParameters(request.method, request.path);
      
      if (parameters == null) {
        continue;
      }

      for (var middleware in this._middlewares) {
        JsonResponse response = Function.apply(middleware, [request]);

        if (response != null) {
          return response;
        }
      }

      return Function.apply(route.handler, [request], parameters);
    }

    return null;
  }
}
