import 'package:pioneer/pioneer.dart';

class Route {
  static Map<String, Converter> converters = {
    'str': Converter(
      'str',
      '[a-zA-Z0-9_-]+',
      (parameter) => parameter
    ),
    'int': Converter(
      'int',
      '[0-9]+',
      (parameter) => int.parse(parameter)
    )
  };

  static void addConverter(Converter converter) {
    Route.converters[converter.type] = converter;
  }

  String method;
  RegExp regexp;
  Handler handler;
  Map<String, Function> casts = {};

  Route(String method, String pattern, Handler handler) {
    var regexp = new RegExp(r'^[0-9a-zA-Z()?:_/-]*$');   
    if (!regexp.hasMatch(pattern)) {
      throw FormatException('Pattern contains illegal symbols. Pattern: ${pattern}.');
    }

    var chunksIn = pattern.split('/');
    var chunksOut = [];
    for (var chunkIn in chunksIn) {
      if (chunkIn.indexOf(':') == -1) {
        chunksOut.add(chunkIn);
        continue;
      }

      var parts = chunkIn.split(':');
      if (parts.length != 2) {
        throw FormatException('Invalid converter usage. Pattern: ${pattern}.');
      }

      var type = parts[0];
      var name = parts[1];

      if (type.length == 0) {
        type = 'str';
      }

      var converter = Route.converters[type];
      if (converter == null) {
        throw FormatException('Invalid converter id. Pattern: ${pattern}.');
      }

      chunksOut.add('(?<' + name + '>' + converter.regexp + ')');
      this.casts[name] = converter.cast;
    }

    this.method = method;
    this.regexp = new RegExp(r'^/' + chunksOut.join('/') + r'$');
    this.handler = handler;
  }

  Map<Symbol, dynamic> matchAndExtractParameters(String method, String path) {
    if (this.method != method) {
      return null;
    }

    RegExpMatch match = this.regexp.firstMatch(path);
    if (match == null) {
      return null;
    }

    if (match.groupCount == 0) {
      return {};
    }

    Map<Symbol, dynamic> result = {};
    for (var name in match.groupNames) {
      var cast = this.casts[name];
      var value = match.namedGroup(name);
      
      result[new Symbol(name)] = cast(value);
    }

    return result;
  }
}
